;/*****************************************************************************\
;*
;*         FILE: TODInit.s
;*      VERSION: 0.2
;*    LAST EDIT: 2009.02.13 01:36:00
;*       AUTHOR: Devia/Ancients
;*
;* ARCHITECTURE: CBM Commodore 64 and compatibles
;*    ASSEMBLER: ca65
;*      OPTIONS: see Makefile
;*
;*  DESCRIPTION: Initialize TOD Clock on all platforms test
;*       STATUS: Under Development
;*         BUGS: NONE
;*
;\*****************************************************************************/

.feature	labels_without_colons

;/*****************************************************************************\
;*** STARTUP *******************************************************************
;\*****************************************************************************/
.segment "STARTUP"

;BASIC header with a SYS call
.proc _HEADER
	.word	Head		;Load address
Head	.word	Next		;Link to next BASIC Line
	.word	2009		;Line number
	.byte	$9E,"2061"	;SYS 2061 - here ---+
	.byte	$00		;End of BASIC line  |
Next	.word	0		;BASIC end marker   |
.endproc			;<------------------+


.proc Start
	sei

	jsr	PALNTSC

	lda	#<STR_Title1
	ldy	#>STR_Title1
	jsr	$ab1e
	lda	#<STR_Title2
	ldy	#>STR_Title2
	jsr	$ab1e
	
	lda	#<STR_TOD
	ldy	#>STR_TOD
	jsr	$ab1e
		
	jsr	TODInit2
	lda	$dc0e
	bmi	tod50
tod60
	lda	#'6'
	sta	Status+1
	lda	#<STR_60
	ldy	#>STR_60
	jmp	:+
tod50
	lda	#'5'
	sta	Status+1
	lda	#<STR_50
	ldy	#>STR_50
:
	jsr	$ab1e

	lda	#<STR_Time
	ldy	#>STR_Time
	jsr	$ab1e

	lda	Status
	sta	STR_Status+13
	lda	Status+1
	sta	STR_Status+14
	lda	Status+2
	jsr	Char2Hex
	sta	STR_Status+15
	stx	STR_Status+16
	lda	Status+3
	jsr	Char2Hex
	sta	STR_Status+17
	stx	STR_Status+18

	lda	#<STR_Status
	ldy	#>STR_Status
	jsr	$ab1e

:	jsr	PrintTime
	jmp	:-
.endproc


;/*****************************************************************************\
;*** SUB ROUTINES **************************************************************
;\*****************************************************************************/
.segment "CODE"

;Initialize TOD Clock by correctly determining the actual frequency
;by which the TOD is clocked.
;Supports PAL/NTSC with 50/60Hz TOD-Clock in ANY combination.
;Theory of Operation:
; 1. Gain max control by setting a dummy NMI and disabling the display
; 2. Set TOD Clock Frequency to 60Hz
; 3. Sync raster to real TOD Clock Frequency
; 4. Count number of CPU cycles it takes for TOD to count 1 decisecond
; 5. If number of cycles counted is less than 118230, TOD Clock Frequency is
;    set to 60Hz, otherwise 50Hz.
;
;Alternatively the same result can be achieved by clocking TOD at 50Hz and
;then just counting 98525, but then the margin for error increases slightly.

.proc	TODInit2
	lda	$01		;Save $01
	pha			; on stack
	lda	#<INT_NMI	;Setup NMI vector
	sta	$fffa		; to catch unwanted NMIs
	lda	#>INT_NMI	;
	sta	$fffb		;
	lda	#$35		;Bank out KERNAL
	sta	$01		; so new NMI vector is active

	lda	#0
	sta	$d011		;Turn off display to disable badlines
	sta	$dc0e		;Set TOD Clock Frequency to 60Hz
	sta	$dc0f		;Enable Set-TOD-Clock
	sta	$dc0b		;Set TOD-Clock to 0 (hours)
	sta	$dc0a		;- (minutes)
	sta	$dc09		;- (seconds)
	sta	$dc08		;- (deciseconds)

	lda	$dc08		;
:	cmp	$dc08		;Sync raster to TOD Clock Frequency
	beq	:-		;
	
	ldx	#0		;Prep X and Y for 16 bit
	ldy	#0		; counter operation
	lda	$dc08		;Read deciseconds
:	inx			;2   -+
	bne	:+		;2/3  | Do 16 bit count up on
	iny			;2    | X(lo) and Y(hi) regs in a 
	jmp	:++		;3    | fixed cycle manner
:	nop			;2    |
	nop			;2   -+
:	cmp	$dc08		;4 - Did 1 decisecond pass?
	beq	:---		;3 - If not, loop-di-doop
				;Each loop = 16 cycles
				;If less than 118230 cycles passed, TOD is 
				;clocked at 60Hz. If 118230 or more cycles
				;passed, TOD is clocked at 50Hz.
				;It might be a good idea to account for a bit
				;of slack and since every loop is 16 cycles,
				;28*256 loops = 114688 cycles, which seems to be
				;acceptable. That means we need to check for
				;a Y value of 28.
	stx	Status+3
	sty	Status+2

	cpy	#28		;Did 114688 cycles or less go by?
	bcc	:+		;- Then we already have correct 60Hz $dc0e value
	lda	#$80		;Otherwise, we need to set it to 50Hz
	sta	$dc0e
:
	lda	#$1b		;Enable the display again
	sta	$d011
	
	pla			;Restore old $01 value
	sta	$01		; and potentially old NMI vector
	rts		
INT_NMI
	rti
.endproc


SCRTMPOS	= $0400+(40*17+32)
.proc PrintTime
	lda	#':'
	sta	SCRTMPOS+2
	ldy	#0
	lda	$dc0a
	jsr	tm
	ldy	#3
	lda	$dc09
	jsr	tm
	rts
tm
	tax
	lsr
	lsr
	lsr
	lsr
	ora	#'0'
	sta	SCRTMPOS,y
	txa
	and	#$0f
	ora	#'0'
	sta	SCRTMPOS+1,y
	rts
.endproc


;Check if PAL or NTSC system
;Returns Carry Clear for NTSC and Carry Set for PAL
.proc PALNTSC
	sei
	lda	$01
	pha
	lda	#$35
	sta	$01
	lda	$fffa
	pha
	lda	$fffb
	pha
	lda	#<INT_NMI
	sta	$fffa
	lda	#>INT_NMI
	sta	$fffb

:	lda	$d011
	bmi	:-
:	lda	$d011
	bpl	:-	
	lda	#<261
:	cmp	$d012
	bne	:-
	ldx	#$ff
:	lda	$d012
	beq	:++
:	cmp	$d012
	beq	:-
	inx
	bpl	:--
:			;X=0: OldNTSC, X=1:NTSC, X=50

	cpx	#50
	beq	:+
	lda	#'N'
	sta	Status
	clc
	bcc	:++
:	lda	#'P'
	sta	Status
	sec
:
	pla
	sta	$fffb
	pla
	sta	$fffa
	pla
	sta	$01
	rts
INT_NMI
	rti
.endproc


;A = Input val
;A = Output Char Hi
;X = Output Char Lo
.proc Char2Hex
	tax
	and	#%11110000
	lsr
	lsr
	lsr
	lsr

	cmp	#9+1
	bpl	letter
	clc
	adc	#$30
	jmp	putch
letter
	clc
	adc	#97-10
putch
	pha
	
	txa
	and	#%00001111

	cmp	#9+1
	bpl	letter2
	adc	#$30
	jmp	putch2
letter2
	clc
	adc	#97-10
putch2
	tax		;Lo part in X
	pla		;Hi part in A
	rts
.endproc

;/*****************************************************************************\
;*** DATA & VARIABLES **********************************************************
;\*****************************************************************************/
.segment "DATA"
STR_Title1	.byte	147,14,13,"-= RELIABLE TOD CLOCK INITIALIZOR 0.2 =-",13
		.byte	"Use a stopwatch to confirm that time is",13
		.byte	"running at correct speed. If it is off ",13
		.byte	"by more than 5 seconds per minute, this",13
		.byte	"routine is probably to blame and you   ",13
		.byte	"should send mail with info about which ",13,0
STR_Title2	.byte	"platform is used to devia@ancients.dk  ",13
		.byte	"Please include:                        ",13
		.byte	"Platform - PAL C64c, NTSC SX-64, etc.  ",13
		.byte	"Power Frequency - 240V/50Hz, 110V/60Hz ",13
		.byte	"Time offset in seconds per minute and  ",13
		.byte	"the Status Code listed below.",13,0
STR_TOD		.byte	13,13,"TOD Clock Frequency: ",0
STR_50		.byte	"50Hz",13,0
STR_60		.byte	"60Hz",13,0
STR_Time	.byte	"Time according to this machine: 00:00",13,0
STR_Status	.byte	"Status Code: xxxxxx",0
Status		.byte	0,0,0,0
.END
